/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)arg_vals.h	1.68	09/05/26 SMI"

#ifndef	_DISKOMIZER_ARG_VALS_H
#define	_DISKOMIZER_ARG_VALS_H

#ifdef	__cplusplus
extern "C" {
#endif

#ifndef ARG_LINK
#define	ARG_LINK(A, B) "<A HREF=\"" A "\">" B "</A> "
#endif

#ifndef ARG_UADMIN_URL
#define	ARG_UADMIN_URL "http://docs.sun.com/app/docs/doc/816-5167/uadmin-2"
#endif

SET_OPENING_BLURB("This page contains all the options that are " \
	"accepted by diskomizer, along with their default values. " \
	"<P>Diskomizer's options can be provided in two ways, in files and " \
	"and on the command line.\n\n<BLOCKQUOTE>-f option_file\n" \
	"<P>The file <B>option_file</B> " \
	"contains a list of all the options.\n" \
	"If the option file does not contain a '/' then option files are " \
	"searched for using the " \
	"<B>DISKOMIZER_PATH</B> environment variable, which is a colon " \
	"separated list of directories to search. " \
	"If it is not set then the current directory and " \
	"then in /opt/CTEdiskomizer/etc are used.\n\n" \
	"<P>-o OPTION=value<P>The options <B>"\
	"OPTION</B> gets set to the value <B>value</B>.</BLOCKQUOTE>"\
	"There can be multiple option files and command line options, " \
	"Diskomizer simply reads them in the order that they are specified." \
	"\n<BR><P>Diskomizer's numeric " \
	"can be postfixed with with various letters to multiply their "
	"values. The following case insensitive letters are understood:\n<UL>"
	"\n\t<LI>C\tCharacters, multiplies by 1."
	"\n\t<LI>B\tBlocks, multiplies by 512."
	"\n\t<LI>K\tKilo bytes, multiplies by 1024."
	"\n\t<LI>M\tMega bytes, multiplies by 1024^2 (1048576)."
	"\n\t<LI>G\tGiga bytes, multiplies by 1024^3 (1073741824)."
	"\n\t<LI>T\tTera bytes, multiplies by 1024^4 (1099511627776)."
	"\n\t<LI>P\tPeta bytes, multiplies by 1024^5 (1125899906842624)."
	"\n\t<LI>E\tExa bytes, multiplies by 1024^6 (1152921504606846976)."
	"\n</UL>"
	"Additionally numeric "
	"options can be given using simple integer expressions to derive " \
	"the value from the default. For example to set the number of " \
	"processes to be 10 times the default do:\n\n<PRE>NPROCS=*10</PRE>\n\n"
	"The following operators are understood:\n<UL>"
	"\n\t<LI>+\t	Add"
	"\n\t<LI>-\t	Subtract"
	"\n\t<LI>*\t	Multiply"
	"\n\t<LI>/\t	Divide"
	"\n\t<LI>%\t	Modulus"
	"\n\t<LI>&\t	And"
	"\n\t<LI>|\t	Or"
	"\n\t<LI>^\t	Xor"
	"\n</UL>"
	"All of the numeric options and any environment variables "
	"are also available as variables in the option file, for "
	"example to set the number of read threads ("
	ARG_LINK("#RTHREADS", "RTHREADS")
	") to be 10 times the number of write threads ("
	ARG_LINK("#WTHREADS", "WTHREADS")
	") do:\n\n<PRE>"
	"RTHREADS= ${WTHREADS} * 10\n</PRE>\nWhen using variables you "
	"should be mindful that the values are substituted as the "
	"variable is encountered, so in this example\n"
	"<PRE>\nWTHREADS=2\nRTHREADS=${WTHREADS} * 10\nWTHREADS=10\n"
	"</PRE>\nwill set RTHREADS to 20 and WTHREADS to 10."
	"\n<P>Lines in the configuration files, including comments, "
	"can be continued onto the next line by including a \\ "
	"character as the last character on the line.\n"
	"<P>\n"
	"The default values are the values that are recommended for general "
	"usage. The default values will change from system to system, as "
	"some are configured based on the system's available resources. "
	"<A HREF=\"#NPROCS\">NPROCS</A> is one such variable that is "
	"defined relative to the number of CPUs in the system. If you "
	"want to know the current default values for the version of "
	"Diskomizer that you are using used:\n"
	"<PRE>diskomizer -d option_file\n</PRE>\n"
	"will write the current values of the options into the file "
	"\"option_file\" in a "
	"format that can be used as an option file to Diskomizer. This example "
	"will dump out the default values, but the -d option can be used "
	"anywhere in the command line and will dump the values as they "
	"are at that point in the argument processing. Multiple -d options "
	"can be given.\n"
	"<PRE>diskomizer -h diskomizer.html\n</PRE>"
	"\nwill create the html documentation in a file called "
	"\"diskomizer.html\"")
START_SECTION(General, "The general options are all you should need to "
	"configure Diskomizer to test disks, meta disks, volumes and files.")
STARG(device, NULL, "A space separated list of the device[s] and or "
	"file[s] to test. You can specify multiple paths to the same device "
	"by grouping  the paths with { and }. Each { and } must be surround "
	"by white space.  If within a group of paths a further group is "
	"specified then the second group is assumed to contain error paths. "
	"These error paths will only be used to replace primary paths if the "
	"error routine FAIL_PATH is used. If you explicitly set the paths "
	"to use then the options "
	"<A HREF=\"#PATHS_TO_USE\">PATHS_TO_USE</A> and "
	"<A HREF=\"#ERROR_PATHS\">ERROR_PATHS</A> are ignored for that device."
	"\nFor example:\n<PRE>DEVICE= { /dev/rdsk/path1 /dev/rdsk/path2 }\n"
	"</PRE>"
	"This will use /dev/rdsk/path1 and /dev/rdsk/path2 as if they are "
	"two paths to the same device.  IO will be load balanced between the "
	"two paths.\n"
	"\n<PRE>DEVICE= { /dev/rdsk/path1 { /dev/rdsk/path2 } }\n"
	"ON_WRITE_ERROR=FAIL_PATH\n"
	"</PRE>Will use /dev/rdsk/path1 until there is a write error and then "
	"use /dev/rdsk/path2 after the error.\n\n"
	"<P><B>If no paths are specified Diskomizer will prompt "
	"you for devices to test.</B></P>")
ILIST(disk_io_sizes, "2K, 8K", "List of disk io sizes, with weightings "
	"in brackets. The higher the weighting the more likely that that io "
	"size will be used. If there is no weighting then the weighting "
	"defaults to 1")
IARG(wthreads, 10, "The number of write threads per device per process.")
IARG(rthreads, 10, "The number of read threads per device per process.")
IARG(wrthreads, 10, "The number of write-read threads per device per process.")
LLARG(file_size, 0LL, "If non zero, the size of the regular files "
		"to be creates. The amount of each raw "
		"device is also limited to this size in bytes.")
IARG(number_of_files, 0, "The number of files to create in each directory "
	"that is specified in the <A HREF=\"#DEVICE\">DEVICE</A> entry. "
	"<A HREF=\"#FILE_SIZE\">FILE_SIZE</A> <B>must</B> also be set.")
BARG(directio, 0, "Use directio if it is available")
LARG(nprocs, sysconf(_SC_NPROCESSORS_ONLN), "The number of processes doing"
	" the io. NOTE, the number of processes created will be one more "
	"than this value. The parent process does no IO.")
#define	stop_after_nloops nloops
LARG(stop_after_nloops, 0L,
	"If there are any <A HREF=\"#RTHREADS\">RTHREADS</a> "
	"or <A HREF=\"#WRTHREADS\">WRTHREADS</a> then this will set "
	"the average number of times a block will be read before the "
	" test completes. If there are no reads, then this will apply to the"
	" number of writes. If this value is zero it means loop forever.")
LLARG(seconds_to_run, 0LL, "The number of seconds to run the test for. "
	"if the value is zero it means run forever, or until some other "
	"event causes Diskomizer to exit.")
LARG(sequential_passes, 1L, "The number of sequential passes to do before "
	"starting to do random writes.")
IARG(open_retries, 1, "The number of times to retry the open of each device"
	" if it fails.")
CARG(start_reads_percentage, 10, "The percentage of the disk that has to have "
	"been written before the first reads are issued. Setting this value "
	"high when <A HREF=\"#EXPERT_READ_MINIMUM\">EXPERT_READ_MINIMUM</A> "
	"is non zero can lead to live lock.")
#define	shared_memory_allocator allocator
STARG(shared_memory_allocator, "BEST",
	"The memory allocator to use to allocate any shared memory. "
	"Values are:\n"
	"<UL>"
	"\n<LI>BEST_SHM Try ISM, and SHM in that order until the memory is"
	" allocated or a hard error is seen, "
	"\n<LI>BEST Try BEST_SHM, and MMAP in that order until the memory is"
	" allocated or a hard error is seen, "
	"\n<LI>ISM Use Intimate shared memory,"
	"\n<LI>SHM Use shared memory, "
	"\n<LI>MMAP Use mmap</UL><P>\n"
	"When using <B>ISM</B> or <B>SHM</B> you will almost certainly have to"
	" increase the values of <B>shmsys:shminfo_shmmax</B>, "
	" <B>shmsys:shminfo_shmmni</B> and <B>shmsys:shminfo_shmseg</B> in "
	" <B>/etc/system</B>. If you change the value of "
	"<B>shmsys:shminfo_shmmax</B> then be sure to set the option "
	"<A HREF=\"#SHMINFO_SHMMAX\"><B>shminfo_shmmax</B></A> to reflect the"
	" new value")
BARG(shm_pageable, 0, "If using the SHM, ISM or BEST_SHM memory allocators "
	"then pass the SHM_PAGEABLE flag to shmget and shmat to allow "
	"the shared memory to be pageable if the operating system supports it")
STARG(checker, "CHECKSUM", "The method of checking to use on the data that is "
	"read of disk. Valid values are:\n<UL>"
	"\n<LI>FULL - Do full checking,"
	"\n<LI>CHECKSUM - Just check the checksums,"
	"\n<LI>READONLY - Just check the checksums when in read-only mode,"
	"\n<LI>NONE - Do no checking</UL>")
IARG(report_time, 10 * MINUTE, "Report on any I/O's that have been outstanding "
	" for this many seconds")
STARG(on_write_error, "RETRY,REWRITE,EXIT",
	"What action to take on a write returns that it has written less bytes "
	"than it was asked to write.  This included the case where the write "
	"returns -1."
	" This is a comma separated list of options."
	" Not all options make sense as anything but the last in the list"
	" Valid values are:\n<UL>"
	"\n<LI><A NAME=\"PAUSE\">PAUSE</A> - Pause and just see if the data "
		"has turned up!,"
	"\n<LI><A NAME=\"RETRY\">RETRY</A> - retry the read asynchronously,"
	"\n<LI>REWRITE - re write the data synchronously,"
	"\n<LI>FAIL_PATH - fail this path and use and alternative if available,"
	"\n<LI>CONTINUE - Continue on,"
	"\n<LI>STOP - stop io to this device,"
	"\n<LI>EXIT - exit,"
	"\n<LI>UADMIN - Issue the "
	    ARG_LINK(ARG_UADMIN_URL, "uadmin(2)")
	    " system call with the arguments "
	    "given by <A HREF=\"#EXPERT_UADMIN_CMD\">EXPERT_UADMIN_CMD</A> "
	    "and <A HREF=\"#EXPERT_UADMIN_FCN\">EXPERT_UADMIN_FCN</A>. Use "
	    "of this option is restricted by the kernel. See "
	    ARG_LINK(ARG_UADMIN_URL, "uadmin(2)")
	    ". This option can be used to quickly stop all io by halting the "
	    "system; allowing state in other devices to be changed as little "
	    "as possible. <B>Use with extreme caution.</B>"
	"\n<LI>ABORT - Abort and dump core, not recommended.</UL>")
STARG(on_error_short, "RETRY,REREAD,EXIT",
	"What action to take on a read returns that it has read less bytes "
	"than it was asked to read.  This included the case where the read "
	"returns -1."
	" This is a comma separated list of options."
	" Not all options make sense as anything but the last in the list"
	" Valid values are:\n<UL>"
	"\n<LI><A NAME=\"PAUSE\">PAUSE</A> - Pause and just see if the data "
		"has turned up!,"
	"\n<LI><A NAME=\"RETRY\">RETRY</A> - retry the read asynchronously,"
	"\n<LI>REREAD - re read the data synchronously,"
	"\n<LI>FAIL_PATH - fail this path and use and alternative if available,"
	"\n<LI>CONTINUE - Continue on,"
	"\n<LI>STOP - stop io to this device,"
	"\n<LI>EXIT - exit,"
	"\n<LI>UADMIN - Issue the "
	    ARG_LINK(ARG_UADMIN_URL, "uadmin(2)")
	    " system call with the arguments "
	    "given by <A HREF=\"#EXPERT_UADMIN_CMD\">EXPERT_UADMIN_CMD</A> "
	    "and <A HREF=\"#EXPERT_UADMIN_FCN\">EXPERT_UADMIN_FCN</A>. Use "
	    "of this option is restricted by the kernel. See "
	    ARG_LINK(ARG_UADMIN_URL, "uadmin(2)")
	    ". This option can be used to quickly stop all io by halting the "
	    "system; allowing state in other devices to be changed as little "
	    "as possible. <B>Use with extreme caution.</B>"
	"\n<LI>ABORT - Abort and dump core, not recommended.</UL>")
STARG(on_error_corrupt, "RETRY,REREAD,EXIT", "What action to take when "
	"data corruption is detected. It takes the same values as "
	"<A HREF=\"#ON_ERROR_SHORT\">ON_ERROR_SHORT</A>.")
STARG(initializer, "SEQ8", "The routine to use to initialise the data "
	"Valid options are: "
	"\n<UL>"
	"\n<LI>RANDOM - Use random data,</LI>"
	"\n<LI>SEQ8 - Use sequential 8 bit data,</LI>"
	"\n<LI>RSEQ8 - Use reverse sequential 8 bit data,</LI>"
	"\n<LI>SEQ16 - Use sequential 16 bit data,</LI>"
	"\n<LI>RSEQ16 - Use reverse sequential 16 bit data,</LI>"
	"\n<LI>SEQ32 - Use sequential 32 bit data,</LI>"
	"\n<LI>RSEQ32 - Use reverse sequential 32 bit data,</LI>"
	"\n<LI>SEQ64 - Use sequential 64 bit data,</LI>"
	"\n<LI>RSEQ64 - Use reverse sequential 64 bit data,</LI>"
	"\n<LI>SUPPLIED_VALUE - Use the value given by </LI>"
	"<A HREF=\"#WRITE_BUFFER_SUPPLIED_VALUE\">WRITE_BUFFER_SUPPLIED_VALUE"
	"</A>, Use of fixed "
	" values is not recommended, unless doing pattern testing, "
	"as this limits the differences in the"
	" data written to each block to the header,</LI>"
	"\n<LI>CJTPAT - Compliant Jitter Tolerance Test Pattern in Fibre "
	"Channel,</LI>"
	"\n<LI>ISIPAT - ISI Killer  Test Pattern in Fibre "
	"Channel,</LI>"
	"\n<LI>ISI_AND_CJT_PAT - ISI Killer  Test Pattern in Fibre and the "
	"ompliant Jitter Tolerance Test Pattern in Fibre Channel one after "
	"another,</LI>"
	"</UL>")
STARG(read_buffer_init, "SUPPLIED_VALUE",
	"The routine to use to initialise read buffers Valid options are:"
	"\n<UL>"
	"\n<LI>NONE - Do no initialisation,"
	"\n<LI>ZERO - Zero out each data buffer before use,"
	"\n<LI>DEADBEEF - Write the bit pattern 0xdeadbeef,"
	"\n<LI>ZERO_ONE - Write the binary bit pattern 01010101,"
	"\n<LI>ONE_ZERO - Write the binary bit pattern 10101010,"
	"\n<LI>XOR - Write opposite bit pattern to the one it expects to read,"
	"\n<LI>SUPPLIED_VALUE - write the value given by <A HREF="
	"\"#READ_BUFFER_SUPPLIED_VALUE\">READ_BUFFER_SUPPLIED_VALUE</A>."
	"\n<LI>DEADBEEF_64 - Write the bit pattern 0xdeadbeef"
	" every 64 bytes"
	"</UL>")
SARG(max_read_retries, 5, "The maximum number of times to retry a read,"
	" when using the <A HREF=\"#RETRY\">RETRY</A> error option")
SARG(max_write_retries, 5, "The maximum number of times to retry a write")
STARG(data_file, NULL, "The name of the directory into which the "
	"configuration is saved")
IARG(read_buffer_supplied_value, 0xfeedbede,
	"A Constant 32 bit quantity to initialise the read buffers with, "
	"when using the SUPPLIED_VALUE option to "
	"<A HREF=\"#READ_BUFFER_INIT\">READ_BUFFER_INIT</a>.")
ILIST(write_buffer_supplied_value,
	"0xffffffff, 0x00000000, 0xffff0000, 0x0000ffff, "
	"0xf0f0f0f0, 0x0f0f0f0f, 0xff00ff00, 0x00ff00ff, "
	"0xfff000ff, 0x0fff000f, 0x00fff000, 0x000fff00, "
	"0xf000fff0, 0xff000fff, "
	"0xAA55AA55, 0x55AA55AA, 0x55555555, 0xAAAAAAAA, "
	"0x3C3C3b3C, 0xC3C3C3C3, 0x66666666, 0x99999999, "
	"0xe77ee77e, 0x7ee77ee7, 0x7e7e7e7e, 0xe7e7e7e7",
	"Constant 32 bit quantities to initialise the write buffers with, "
	"when using the SUPPLIED_VALUE option to "
	"<A HREF=\"#INITIALIZER\">INITIALIZER</a>. Each value is used "
	"to initialize a buffer until all the values have been used "
	"and then the first value is used again. Weightings can "
	"be applied to the values so that some values are used more "
	"then others.")
BARG(fast_start, 1, "If non zero do not do anything during the initialisation"
	" that can be left until run time.  The down side of this is that "
	" if short of space in the directory specified by "
	"<A HREF=\"#EXPERT_MMAP_FILE_DIRECTORY\">EXPERT_MMAP_FILE_DIRECTORY</A>"
	" Diskomizer can get a SIGBUS at runtime, which it will catch and "
	" report as a memset error.")
CARG(paths_to_use, 2, "The number of alternate paths to use in parallel"
	" when available. See also <A HREF=\"#DEVICES\">DEVICES</A>")
CARG(error_paths, 0, "The number of alternate paths to have open and use in "
	"case of errors, when available. "
	"See also <A HREF=\"#DEVICES\">DEVICES</A>")
IARG(how_often_to_report, 10000, "After this number of i/o requests report the "
	"times, Times are reported at LOG_INFO priority. If this is set to "
	"zero then never report.")
BARG(use_long_names, 0, "Use long names of devices when displaying them"
	" if using a Veritas volumes or Solstice Disk Suite"
	" devices set this to 1")
STARG(STDOUT, NULL, "Use this file as standard out. The file will be appended"
	" to.")
STARG(STDERR, NULL, "Use this file as standard error. The file will be appended"
	" to.")
STARG(stderr_priority, "WARNING",
	"Messages at this priority and above will go into "
	"the <A HREF=#STDERR>STDERR</A> file. "
	"The priorities used are syslog priorities.")
STARG(stdout_priority, "INFO", "Messages at this priority and above and below "
	"<A HREF=#stderr_priority>STDERR_PRIORITY</A> will go into the the "
	"<A HREF=#STDOUT>STDOUT</A> file")
BARG(background, 0, "If non zero and stdout and stderr are set"
	" Diskomizer will background itself.")
STARG(workingdir, "/tmp/diskomizer", "The base name of the working "
	"directory for the processes.")
LARG(shminfo_shmmax, 0x100000L, "The maximum size of a shared memory segment."
	" This should be set to be the same as the setting of "
	"<B>shmsys:shminfo_shmmax</B> in <B>/etc/system</B>. The default"
	" value matches the default value of <B>shmsys:shminfo_shmmax</B>"
	" in SunOS 5.7.")
BARG(o_rdonly, 0, "If set non zero open the file[s] O_RDONLY. "
	"This makes the write threads into special read threads which "
	"if the block has not yet been read, just "
	"read the block and calculate the checksum based on the current "
	"content of the block. If the block has been read then the checksum "
	"is compared with the previously calculated checksum and and "
	"difference reported.\nNo diffs files are produced in read-only mode.\n"
	"The I/O size is constant in this mode, the largest I/O size "
	"specified by <A HREF=#DISK_IO_SIZES>DISK_IO_SIZES</A> is used for "
	"all I/O. The only values of <A HREF=#CHECKER>CHECKER</A> that "
	"are legal are:\n<UL>"
	"\n<LI>READONLY - Diskomizer checks the data using checksums. "
	"\n<LI>NONE - Diskomizer does no checking."
	"\n</UL>")
BARG(o_sync, 0, "If set non zero open the file[s] O_SYNC. Use this option when"
	" testing files in a file system, rather than raw devices.")
BARG(display_prev_diffs, 0, "If non zero then display the diffs for the "
	"previous write to the block as well as the diffs for the last write "
	"to the block.")
STARG(syslog_facility, NULL,
	"The facility to use if you wish to use syslog to log "
	"messages. To put useful messages into <I>/var/adm/messages</I> "
	"on a standard installation set this to <B>DAEMON</B>.")
STARG(syslog_log_upto_priority, "NOTICE",
	"The syslog priority up to which messages will be logged when "
	"using syslog. This should be set to the highest priority "
	"of message that you wish to have logged to syslog."
	"Setting this too low results in syslogd being the bottleneck "
	"for the number of ios that can be generated.")
STARG(aio_routines, "SUNOS", "Routines for doing async io."
	" Some possible values are:\n"
	"<UL>"
	"\n<LI>SUNOS - Use the sunos aio routines "
		"(aioread and aiowrite).</LI>"
	"\n<LI>POSIX - Use the POSIX aio routines "
		"(aio_read and aio_write).</LI>"
	"\n<LI>PREAD - Use pread(2) and pwrite(3) and "
		"threads to simulate aio.</LI>"
	"</UL>")
END_SECTION(General)
/*
 * Expert options.
 */
START_SECTION(Expert, "The expert options are not intended for general "
	"use. A number of them are there to tune the performance of "
	"Diskomizer to get it to be able to Diskomize more disks, "
	"particularly when the address space is limited.")
#define	expert_nlocks nlocks
IARG(expert_nlocks, 0, "The number of locks used to protect the bitmaps")
LARG(expert_release_read_buffers_after_n_uses, 10L, "The number of times "
	"each read buffer is used before being release back into the pool. ")
LARG(expert_release_write_buffers_after_n_uses, 10L, "The number of times "
	"each write buffer is used before being release back into the pool. ")
#define	expert_number_of_write_buffers nwritebufs
LARG(expert_number_of_write_buffers, 0L,
	"The number of write buffers to use. A value of zero causes "
	"the system to choose the number of buffers to use. The number "
	"chosen by default is 10 times the maximum that will be required "
	"at any given time. This makes searching for a free buffer quick "
	"but when there are large numbers of devices and processes and "
	"writing threads makes the memory usage very large.  Setting this "
	"value to 1 will make the system choose a value that is 1.1 times "
	"the maximum number of blocks required.")
#define	expert_number_of_read_buffers nreadbufs
LARG(expert_number_of_read_buffers, 0L,
	"The number of read buffers to use. A value of zero causes "
	"the system to choose the number of buffers to use. See "
	"<A HREF=\"#expert_number_of_write_buffers\">"
	"expert_number_of_write_buffers</A> for details of memory usage.")
#define	expert_start_offset start_offset
LARG(expert_start_offset, 4L,
	"The offset in Diskomizer blocks at which to start"
	" reading and writing, this can be used to protect the disk label")
#define	expert_save_data save_data
BARG(expert_save_data, 0, "If non zero save the buffers that are going to be "
	"used to disk before starting")
#define	expert_init_from_data_file init_from_data_file
BARG(expert_init_from_data_file, 0,
	"Initialize buffers from a file created by "
	"the <A HREF=\"#EXPERT_SAVE_DATA\">EXPERT_SAVE_DATA</A> option.")
#define	expert_fix_bad_checksums fix_bad_checksums
BARG(expert_fix_bad_checksums, 0,
	"If non zero it will fix bad checksums if it "
	"finds any after initialising the data buffers.")
LARG(expert_mmap_max_size, sysconf(_SC_PAGESIZE) * sizeof (struct blks),
	"For mmap memory that can be allocated in chunks, the largest chunk "
	"size.")
#define	expert_hint_shm_chunk_size shm_max_size
LARG(expert_hint_shm_chunk_size, sysconf(_SC_PAGESIZE) * sizeof (struct blks),
	"For shm memory that can be allocated in chunks, the largest chunk "
	"size.")
#define	expert_mmap_file_directory mmap_file_directory
STARG(expert_mmap_file_directory, "/tmp",
	"The directory in which to create the "
	"shared memory mmap files. This directory should not be remote, "
	"in fact if it is not tmpfs you will only end up testing this "
	"directory and not the devices. It will need to have a lot of "
	"space in it.")
BARG(expert_always_mmap_from_file, 0, "If non zero then never map from "
	"/dev/zero, always use the files")
#define	expert_read_buffer_gap read_buffer_gap
IARG(expert_read_buffer_gap, 0,
	"Size in bytes of of the gap between read buffers. "
	"This leads to more exciting alignments and buffers that cross page "
	"boundaries")
#define	expert_write_buffer_gap write_buffer_gap
IARG(expert_write_buffer_gap, 0,
	"Size in bytes of of the gap between write buffers. "
	"This leads to more exciting alignments and buffers that cross page "
	"boundaries")
#define	expert_read_minimum read_minimum
IARG(expert_read_minimum, 0, "The minimum number of times a block must be read"
	" before it is rewritten. If not enough reads are done this can lead "
	"to Diskomizer deferring writes while it finds blocks that have "
	"been written.")
LLARG(expert_min_active_time, LONGLONG_MAX,
	"The minimum time in micro seconds that the device "
	"will be kept busy before all i/o is allowed to drain and the device "
	"is allowed to be idle")
LLARG(expert_max_active_time, 0LL,
	"The maximum time in micro seconds that the device "
	"will be kept busy before all i/o is allowed to drain and the device "
	"is allowed to be idle, zero indicates there will be no idle times.")
LLARG(expert_min_idle_time, 0LL,
	"The minimum time in micro seconds to pause for while the disk "
	"drive is to be idle")
LLARG(expert_max_idle_time, 0LL,
	"The maximum time in micro seconds to pause for while the disk "
	"drive is to be idle")
IARG(expert_read_cluster_length, 1, "The length of a read i/o cluster. "
	"This must never be zero")
IARG(expert_write_cluster_length, 1, "The length of a write i/o cluster. "
	"This must never be zero")
BARG(expert_small_diffs, 0, "If non zero, then when printing diffs files "
	"only print the lines that contain differences.")
BARG(expert_cleanup_created_files, 1, "If non zero, then any files that are "
	"created to do io too and from are removed at the end of the run. "
	"Files that are not created by Diskomizer are not removed. "
	"If there are any errors during the run the files are not deleted "
	"so you can perform a post mortem.")
CARG(expert_decimal_places, 3, "The number of decimal places to report "
	"the io times to.")
LLARG(expert_amount_to_leave_unused, 0LL, "The number of bytes to leave unused "
	"in a directory. This is not used when doing io to raw or "
	"block devices")
SARG(expert_recent_log_size, 0, "The size of the \"recent write\" log. "
	"The offsets of the last expert_recent_log_size successful writes "
	"by each process on each device are logged, so that if there is an "
	"event they can quickly be checked for problems.")
STARG(expert_uadmin_cmd, "A_SHUTDOWN", "Default command for on error UADMIN "
	"see "
	ARG_LINK(ARG_UADMIN_URL, "uadmin(2)")
	" for a complete list of options on this system.")
STARG(expert_uadmin_fcn, "AD_HALT", "Default function for on error UADMIN "
	"see "
	ARG_LINK(ARG_UADMIN_URL, "uadmin(2)")
	" for a complete list of options on this system.")
STARG(expert_uadmin_mdep, NULL, "Default string to pass as mdep "
	"for on error UADMIN. See "
	ARG_LINK(ARG_UADMIN_URL, "uadmin(2)")
	" for a complete list of options on this system.")
BARG(expert_allow_detach, 1, "Allow the detaching of shared memory segments.")
STARG(expert_userpat_file, "userpat_data_file", "Name of the file contianing"
	"the data to initialise the write buffers with. This is binary data"
	"that is repeated across all the buffers")
BARG(expert_do_path_check, 1, "Check of incorrect configuration where the same "
	"device or file has been listed more than once")
END_SECTION(Expert)
/*
 * End of the Expert options.
 */
/*
 *	The obscure options. I kid you not...
 */
START_SECTION(Obscure, "The Obscure options are options that have either "
	"been added for specific problems, or to change the behaviour "
	"of the program to do the unusual or for tuning the program. "
	"If the option was added for tuning then the assumption is that "
	"it's default value is now well understood and gives the desired "
	"level of performance.")
BARG(obscure_execute, 0,
	"If non zero then execute code that is read from the "
	"device if it passes the check and is marked executable. This "
	"is here to test executing instructions directly from shared memory.")
#define	obscure_max_fork_failure max_fork_failure
CARG(obscure_max_fork_failure, 5, "The number of times a fork will be "
	"tried before giving up. Fork return EAGAIN so it is reasonable"
	" to have a few goes before giving up.")
#define	obscure_fork_failure_wait_time fork_failure_wait_time
CARG(obscure_fork_failure_wait_time, 20, "The number of seconds to wait "
	"between each fork failure before trying again.")
#define	obscure_use_fork1 use_fork1
BARG(obscure_use_fork1, 0,
	"If set non zero it will use fork1 instead of fork.")
#define	obscure_keep_shm keep_shm
BARG(obscure_keep_shm, 0, "If non zero the shared memory segments are not "
	"immediately deleted when they can be. Why you would want to keep "
	"the shared memory segments is beyond me, but this option has always "
	"been there so it stays.")
#define	obscure_max_gettimeofday_retries max_gettimeofday_retries
CARG(obscure_max_gettimeofday_retries, 100,
	"The maximum number of times to retry "
	"gettimeofday when it returns bogus usec values. This is a work around"
	" for bug 4126459.")
#define	obscure_o_ndelay o_ndelay
BARG(obscure_o_ndelay, 0, "If set non zero open the file[s] O_NDELAY")
#define	obscure_o_trunc o_trunc
BARG(obscure_o_trunc, 0, "If set non zero open the file[s] O_TRUNC")
#define	obscure_o_excl o_excl
BARG(obscure_o_excl, 1, "If set non zero open the file[s] O_EXCL, "
	"do not ever set this to zero.")
#define	obscure_pause_time pause_time
IARG(obscure_pause_time, 5, "The time to pause for when using the "
	"<A HREF=\"#PAUSE\">PAUSE</A> error handler.")
#define	obscure_max_mem_delay max_mem_delay
IARG(obscure_max_mem_delay, 0,
	"The maximum number of cycles to wait before checking "
	"that the checksum written into shared memory is correct. This is"
	" an attempt to check for data corruption in shared memory")
#define	obscure_use_madvise use_madvise
BARG(obscure_use_madvise, 0,
	"If not zero set call madvise when getting and returning "
	" buffers. Use of madvise could make things faster, or can be used "
	" to just be cruel to the VM system, by saying we don't want things "
	"just before we use them, or saying we need things when we have just"
	" finished with them.  Added in an attempt to reproduce a specific "
	" problem, instead was able to log bug 4246624.")
#define	obscure_madvise_get madvise_get
CARG(obscure_madvise_get, MADV_WILLNEED,
	"If <A HREF=\"#OBSCURE_USE_MADVISE\">OBSCURE_USE_MADVISE</A> is set,"
	"then use this value as the value for the advice flag to madvise"
	" when getting an io buffer.")
#define	obscure_madvise_return madvise_return
CARG(obscure_madvise_return, MADV_DONTNEED,
	"If <A HREF=\"#OBSCURE_USE_MADVISE\">OBSCURE_USE_MADVISE</A> is set,"
	" then use this value as the value for the advice flag to madvise"
	" when returning an io buffer.")
BARG(obscure_leave_shm_attached_after_init, 0, "After creating shared memory "
	"leave the memory attached. Otherwise all the memory segments that can "
	"be detached are detached and then attached as required.")
LARG(obscure_writemap_size, 0L,
	"The size in bytes of the bitmap that protects the disk blocks. "
	"Use of this options is <B>strongly</B> discouraged, use in "
	"conjunction "
	"with <A HREF=\"#EXPERT_READ_MINIMUM\">EXPERT_READ_MINIMUM</A> can "
	"easily lead to Diskomizer thrashing and deferring writes.")
LARG(obscure_readmap_size, 0L,
	"The size in bytes of the bitmap that protects the read buffers. "
	"Use of this option is <B>strongly</B> discouraged, any saving in "
	"space is insignificant compared to the amount of space used for "
	"the actual buffers.")
LARG(obscure_search_multiplier, 1000L, "Multiplier to effect how long to "
	"search for blocks to write before deferring the write. The larger "
	"the number the less time to search.  Setting this to 0 will lead to "
	"diskomizer searching forever, which is a live lock. This is only "
	"used when <A HREF=\"#EXPERT_READ_MINIMUM\">EXPERT_READ_MINIMUM</A> "
	"is set.")
STARG(obscure_data_file_basename, "DISKOMIZER_TEST_TARGET_", "The name of any "
	"data files that get created by diskomizer when "
	"<A HREF=\"#NUMBER_OF_FILES\">NUMBER_OF_FILES</A> is non zero and a "
	"directory is given as the target <A HREF=\"#DEVICE\">DEVICE</A>"
	" for the test.")
BARG(obscure_mprotect_write_bufs, 0, "Set this non zero to call mprotect "
	"to toggle the protections on the buffers used for writing. "
	"Has severe performance consequences.")
IARG(obscure_buffer_multiplier, 2, "The amount by which to multiply the "
	"the number of read and write buffers by.")
BARG(obscure_report_priority, 1, "If non zero then report the syslog priority "
	"of any messages that are printed.")
CARG(obscure_bit_percentage, 1, "The maximum percentage of bits that can be "
	"corrupted and diskomizer still report the corruption as bit "
	"corruption")
BARG(obscure_raw_bits, 0, "If non zero print all bits that are in error in "
	"raw form no matter how many are in error otherwise for small numbers "
	"of bits in error it will list the bit offsets.")
BARG(obscure_allow_efi_102, 0, "Allow the opening of devices that have an"
	" EFI label with a version of 0x00010002. These were written by the "
	" format command on Solaris prior to fixing bug 4888423")
STARG(obscure_usagetracking_domain, ".sun.com", "Domain within which to do"
	" usage tracking. Set this to \"\" to disable usage tracking.")
STARG(obscure_usage_email, "pts_usage_tracking+diskomizer@sun.com",
	"Email address to send usage tracking to, only used if the current "
	"domain is a sub domain of "
	ARG_LINK("#OBSCURE_USAGETRACKING_DOMAIN",
		"OBSCURE_USAGETRACKING_DOMAIN") ".")
STARG(obscure_usage_file, "usage_tracking.xml",
		"Name of file to store the usage tracking "
		"information in.")
STARG(obscure_sendmail, "/usr/lib/sendmail", "Program to deliver email for "
		"usage tracking.")
BARG(obscure_allow_non_zero_efi_reserved, 1, "If true then do not treat non "
		"zero entries in the efi label's reserved areas as errors")
END_SECTION(Obscure)
/*
 * End of obscure options.
 */
/*
 * Start of debug options.
 */
START_SECTION(Debug, "These options are only used when debugging diskomizer. "
	" Their use is strongly discouraged.")
BARG(debug, 0, "If non zero print debug messages, not recommended.")
BARG(debug_no_action, 0, "Exit after parsing all the options and opening all "
	"the devices and checking for duplicate paths but not doing the main "
	"io")
BARG(debug_always_detach, 0,
	"If set non zero memory that can be detached will"
	" always be detached and reattached when needed. The default is "
	"only to detach memory when short of address space.")
BARG(debug_allow_coredump, 0,
	"Set this non zero if you want to get core files "
	"when there is a failure")
BARG(debug_pause_on_signal, 0,
	"Set this non zero and diskomizer will pause to allow a debugger to "
	"attach on a fatal error.")
BARG(debug_report_errors_as_discovered, 0,
	"Report errors when the code discovers them rather than storing the "
	"error information and reporting it in a consise manner.")
#define	debug_dev_tree dev_tree
STARG(debug_dev_tree, "/devices", "path to a /devices tree.")
END_SECTION(Debug)

/* Just to keep hdrchk happy. This header must have no guard */
#undef	_DISKOMIZER_ARG_VALS_H

#ifdef	__cplusplus
}
#endif

#endif	/* _DISKOMIZER_ARG_VALS_H */
