/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)args.h	1.28	09/05/26 SMI"

#ifndef _DISKOMIZER_ARGS_H
#define	_DISKOMIZER_ARGS_H

#ifdef __cplusplus
extern "C" {
#endif

#define	LONGLONG_MAX 0x7fffffffffffffffLL
#include <ctype.h>
#include <stdio.h>
#include <sys/types.h>
#define	KILO_BYTE(A) (1024*A)
#define	MEGA_BYTE(A) (1024*1024*A)
#define	DAYS(A) (24*3600*A)
#define	MINUTE 60
#define	MINS(A) (MINUTE * A)
struct ilist {
	int wlen;
	int *vals;
	int *weightings;
};
/*
 * All these defines are done do the structure has all the 64 bit
 * quantites, then the 32 bit quantities then the 8 bit quantities.
 * so that we don't waste space with too much padding, but the order
 * of the file arg_vals.h defines the order in which things get printed
 */
#define	START_SECTION(A, B)
#define	END_SECTION(A)
#define	SET_OPENING_BLURB(A)
struct options {
#define	ILIST(B, C, D) struct ilist B;
#define	STARG(B, D, C)
#define	SARG(B, D, C)
#define	LARG(B, D, C)
#define	IARG(B, D, C)
#define	CARG(B, D, C)
#define	BARG(B, D, C)
#define	LLARG(B, D, C)
#include "arg_vals.h"
#undef ILIST
#undef LLARG
#define	ILIST(A, B, C)
#define	LLARG(B, D, C) long long B;
#include "arg_vals.h"
#undef	LLARG
#define	LLARG(B, D, C)
#undef	STARG
#undef	LARG
#define	STARG(B, D, C) char *B;
#define	LARG(B, D, C) ulong_t B;
#include "arg_vals.h"
#undef STARG
#undef LARG
#undef LLARG
#undef IARG
#define	STARG(B, D, C)
#define	LLARG(B, D, C)
#define	LARG(B, D, C)
#define	IARG(B, D, C) int B;
#include "arg_vals.h"
#undef IARG
#define	IARG(B, D, C)
#undef	SARG
#define	SARG(B, D, C) ushort_t B;
#include "arg_vals.h"
#undef SARG
#define	SARG(B, D, C)
#undef CARG
#define	CARG(B, D, C) uchar_t B;
#include "arg_vals.h"
#undef CARG
#undef BARG
#define	CARG(B, D, C)
#define	BARG(B, D, C) uint_t B:1;
#include "arg_vals.h"
#undef STARG
#undef SARG
#undef IARG
#undef LLARG
#undef LARG
#undef CARG
#undef BARG
#undef ILIST
};
#undef START_SECTION
#undef END_SECTION
#undef SET_OPENING_BLURB

extern struct options opts;
#define	OPTION(A) opts.A

/*
 * Option Store.
 *
 *	The option store can be used by a shared libary so that the library
 *	can access options.
 *
 *	The library calls opts_init which returns a pointer to an option_opts
 *	structure. Each of the option opts takes 2 arguments, the first being
 *	the key and the second a pointer to the variable into which to write
 *	the result. They all return OPT_OK on success and OPT_FAILED on
 *	failure.
 *
 *	On success it writes the value into the variable pointed to by the
 *	second argument.
 *
 *	Options should be processed at the start of the program and cached
 *	as the option store is not "super fast". Once all the options are
 *	processed opts_fini() should be called and then option routines
 *	should not be used again.
 *
 *	Example:
 *
 *	char *name;
 *	int count;
 *	struct option_ops *ops;
 *
 *	ops = opts_init();
 *	if (ops->opt_str("TEST_OPTION", &name) == OPT_OK) {
 *		name = strdup(name);
 *	}
 *	if (ops->opt_int("TEST_INT", &test_int) != OPT_OK) {
 *		fprintf(stderr, "Unknown option \"TEST_INT\"\n");
 *	}
 *
 */
enum option_status {
	OPT_FAILED = 0,
	OPT_OK = 1
};
typedef enum option_status option_status_t;

struct option_ops {
	option_status_t (*opt_bool)(const char *, char *);
	option_status_t (*opt_char)(const char *, char *);
	option_status_t (*opt_short)(const char *, short *);
	option_status_t (*opt_int)(const char *, int *);
	option_status_t (*opt_long)(const char *, long *);
	option_status_t (*opt_long_long)(const char *, long long *);
	option_status_t (*opt_str)(const char *, char **);
};

extern const struct option_ops *opts_init(void);
extern int opts_fini(void); /* returns a count of the unused options */
extern int do_args(int argc, char **argv, void (*pprintf)(char *fmt, ...),
	const char *path);
extern void print_args(int argc, char **argv,
	void (*pprintf)(const char *fmt, ...));
extern int print_options(FILE *f);
extern void usage(const char *prog);
extern char *map_file(char *file, time_t *last_readp, size_t *lenp,
	void (*pprintf)(char *fmt, ...), const char *path);
extern int save_options(const char *file);
extern int read_config_file(char *file, void (pprintf)(char *, ...),
	const char *path);
extern int load_ilist(struct ilist *ilp, int *vals, int len);
extern const char *set_diskomizer_path(void);
extern const struct option_ops option_ops;
#ifdef __cplusplus
}
#endif

#endif /* _DISKOMIZER_ARGS_H */
