/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

#pragma ident	"@(#)asm.c	1.12	09/05/26 SMI"

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 *
 *	routines for writing SPARC V9/Intel machine code. All could be macros
 *	but speed is not that important to me when this code is run
 *
 *	See "The SPARC Architecture Manual" and
 *  "Intel Architecture Software Developer's Manual" for more info.
 */

#include <sys/types.h>
/*LINTED*/
#include "asm.h"

#if __sparc
#define	N5BITS 0x1f
#define	N13BITS 0x1fff
#define	N22BITS 0x3fffff

uint_t
add_asm(ushort_t rd, ushort_t rs1, char i, ushort_t x)
{
	uint_t ret;
	ret = ((uint_t)0x2 << 30) | ((rd & N5BITS) << 25) |
	    ((rs1 & N5BITS) << 14) | ((0x1 & i) << 13) | (N13BITS & x);
	return (ret);
}
uint_t
sllx_asm(ushort_t rd, ushort_t rs1, ushort_t cnt)
{
	uint_t ret;
	ret = ((uint_t)1 << 31) | (rd << 25) | (0x25 << 19) |
	    (rs1 << 14) | (0x3 << 12) | (1 << 13) | (N13BITS & cnt);
	return (ret);
}
uint_t
sethi_asm(ushort_t const22, ushort_t rd)
{
	uint_t ret;

	ret = ((N5BITS & rd) << 25) | (0x4 << 22) | (N22BITS & const22);
	return (ret);
}
uint_t
jmpl_asm(ushort_t rd, ushort_t rs1, char i, ushort_t x)
{
	uint_t ret;

	ret = ((uint_t)1 << 31) | (rd << 25) | (0x38 << 19) | (rs1 << 14) |
	    (i << 13) | (N13BITS & x);
	return (ret);
}
uint_t
bn_asm(char a, uint_t x)
{
	uint_t ret;

	ret = (a << 29) | (0x2 << 22) | (N22BITS & x);
	return (ret);
}
uint_t
ba_asm(char a, uint_t x)
{
	uint_t ret;

	ret = (a << 29) | (0x8 << 25) | (0x2 << 22) | (N22BITS & x);
	return (ret);
}
uint_t
save_asm(ushort_t rd, ushort_t rs1, char i, ushort_t x)
{
	uint_t ret;

	ret = ((uint_t)1 << 31) | (rd << 25) | (0x3c << 19) | (rs1 << 14) |
	    (i << 13) | (N13BITS & x);
	return (ret);
}
uint_t
restore_asm(ushort_t rd, ushort_t rs1, char i, ushort_t x)
{
	uint_t ret;

	ret = ((uint_t)1 << 31) | (rd << 25) | (0x3d << 19) | (rs1 << 14) |
	    (i << 13) | (N13BITS & x);
	return (ret);
}
#elif __i386 /* Intel */ || __amd64

/*
 * Sadly, due to the varying instruction size inherent with CISC
 * the assembler generated will be interspersed with NOPs. Padding
 * things out like this seems like the simplest way to fit in with
 * the current code which assumes easy multiples of 32 bits.
 */


#ifdef __amd64

varlen
mov32_asm(uint32_t imm32, ushort_t reg)
{
	varlen ret;

	ret.thirtytwo[0] = ((uint32_t)(0xb8+reg) << 24) | 0x909090;
	ret.thirtytwo[1] = imm32;

	return (ret);
}
varlen
movl_asm(uint64_t imm64, ushort_t reg)
{
	varlen ret;

	ret.thirtytwo[0] = ((uint32_t)(0xb8+reg) << 24) | 0x489090;
	ret.thirtytwo[1] = (uint32_t)LOWER32(imm64);
	ret.thirtytwo[2] = (uint32_t)UPPER32(imm64);

	return (ret);
}

varlen
pushl_asm(uint32_t imm32) /* pushl <imm64> */
{
	varlen ret;

	ret.thirtytwo[0] = (uint32_t)0x68909090;
	ret.thirtytwo[1] = imm32;

	return (ret);
}

varlen
jmp_asm(uint32_t offset)
{
	varlen ret;

	ret.thirtytwo[0] = (uint32_t)0xe9909090;
	ret.thirtytwo[1] = offset;
	ret.thirtytwo[2] = 0;

	return (ret);
}
#else

varlen
movl_asm(uint32_t imm32, ushort_t reg)
{
	varlen ret;

	ret.thirtytwo[0] = ((uint32_t)(0xb8+reg) << 24) | 0x909090;
	ret.thirtytwo[1] = imm32;

	return (ret);
}

varlen
pushl_asm(uint32_t imm32) /* pushl <imm32> */
{
	varlen ret;

	ret.thirtytwo[0] = (uint32_t)0x68909090;
	ret.thirtytwo[1] = imm32;

	return (ret);
}

varlen
jmp_asm(uint32_t offset)
{
	varlen ret;

	ret.thirtytwo[0] = (uint32_t)0xe9909090;
	ret.thirtytwo[1] = offset;

	return (ret);
}
#endif

#else
#error "Unkown processor"
#endif
