/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)asm.h	1.11	09/05/26 SMI"

#ifndef _DISKOASM_H
#define	_DISKOASM_H

#ifdef	__cplusplus
extern "C" {
#endif

#ifdef __sparc
#define	hi(A) (A >> 10)
#define	lo(A) (A & 0x3ff)
/*
 * All the registers.
 */
#define	g0 0
#define	g1 1
#define	g2 2
#define	g3 3
#define	g4 4
#define	g5 5
#define	g6 6
#define	g7 7
#define	o0 8
#define	o1 9
#define	o2 10
#define	o3 11
#define	o4 12
#define	o5 13
#define	o6 14
#define	o7 15
#define	l0 16
#define	l1 17
#define	l2 18
#define	l3 16
#define	l4 20
#define	l5 21
#define	l6 22
#define	l7 23
#define	i0 24
#define	i1 25
#define	i2 26
#define	i3 27
#define	i4 28
#define	i5 29
#define	i6 30
#define	i7 31
#define	sp o6
#define	fp i6
/*
 * The synthetic instructions are just macros
 */
#define	ret_asm() jmpl_asm(g0, i7, 1, 0x8)
#define	retl_asm() jmpl_asm(g0, o7, 1, 0x8)
/*
 * All the instructions this program needs to be able to write.
 */
#define	nop_asm() 0x01000000
uint_t add_asm(ushort_t rd, ushort_t rs1, char i, ushort_t x);
uint_t sllx_asm(ushort_t rd, ushort_t rs1, ushort_t cnt);
uint_t sethi_asm(ushort_t const22, ushort_t rd);
uint_t jmpl_asm(ushort_t rd, ushort_t rs1, char i, ushort_t x);
uint_t ba_asm(char a, uint_t x);
uint_t bn_asm(char a, uint_t x);
uint_t save_asm(ushort_t rd, ushort_t rs1, char i, ushort_t x);
uint_t restore_asm(ushort_t rd, ushort_t rs1, char i, ushort_t x);

#else /* Intel */

/*
 * Registers, values as per the +rd nomenclature.
 */

#define	eax ((ushort_t)0)
#define	ecx ((ushort_t)1)
#define	edx ((ushort_t)2)
#define	ebx ((ushort_t)3)
#define	esp ((ushort_t)4)
#define	ebp ((ushort_t)5)
#define	esi ((ushort_t)6)
#define	edi ((ushort_t)7)

#define	rax ((ushort_t)0)
#define	rdx ((ushort_t)2)

#define	NOP	((ushort_t)0x90) /* NOP opcode */
#define	NOPWORD ((uint32_t)0x90909090)

#define	nop_asm() NOPWORD
#if defined(__amd64)
#define	save_asm()  ((uint32_t)0xe5894855)
#define	call_asm()  ((uint32_t)0xd2ff9090)
#define	ret_asm()   ((uint32_t)0xc3c99090)
#else
#define	ret_asm()   ((uint32_t)0xc3c99090)
#define	save_asm()  ((uint32_t)0xec8b5590)
#define	call_asm()  ((uint32_t)0xd0ff9090)
#endif
#define	add_asm(ib, reg) \
	((uint32_t)0x9000c083) | ((uint32_t)reg << 8) | ((uint32_t)ib << 16)

/*
 * Handy bitmasks, specifically verbose for clarity.
 */

#define	BITS24_31 ((uint32_t)0xFF000000)
#define	BITS16_23 ((uint32_t)0x00FF0000)
#define	BITS8_15 ((uint32_t)0x0000FF00)
#define	BITS0_7 ((uint32_t)0x000000FF)

#define	UPPER32(A) (A >> 32)
#define	LOWER32(A) (A & 0xffffffff)

/*
 * Specifically make varlen stand out from the tmp variable
 * used in some of the buf routines simply for clarity. This
 * variable is required as some of the Intel assembler is
 * greater than 32 bits but, for ease of porting it is
 * handy to still handle 32 bit quantities.
 */

#if defined __amd64

typedef union {
	uint64_t	sixtyfour;
	uint32_t	thirtytwo[3];
} varlen;

extern varlen mov32_asm(uint32_t imm32, ushort_t reg);
extern varlen movl_asm(uint64_t, ushort_t);
extern varlen pushl_asm(uint32_t);
extern varlen jmp_asm(uint32_t);
#elif defined __i386

typedef union {
	uint64_t	sixtyfour;
	uint32_t	thirtytwo[2];
} varlen;

extern varlen movl_asm(uint32_t, ushort_t);
extern varlen pushl_asm(uint32_t);
extern varlen jmp_asm(uint32_t);
#else
#error "unknown processor type
#endif

#endif /* Intel */

#ifdef	__cplusplus
}
#endif

#endif /* _DISKOASM_H */
