/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)bufs.h	1.31	09/05/26 SMI"

#ifndef _DISKOMIZER_BUFS_H
#define	_DISKOMIZER_BUFS_H

#ifdef	__cplusplus
extern "C" {
#endif

typedef struct {
	uint_t BUF_EXECUTABLE : 1;
	uint_t BUF_BAD_HDR : 1;
	uint_t BUF_BAD_CHKSUM : 1;
	uint_t BUF_READ_ONLY : 1;
	uint_t BUF_READY : 1; /* If not zero then it needs initalizing */
	/* Write sequence number. The same size as in the blks structure */
	uint_t sequence: SEQUENCE_BITS;
	/*
	 * the spare bits we have, the 5 is the count of those sinle bits
	 * above
	 */
	uint_t padding: (WORD_BIT - 5) - SEQUENCE_BITS;
} buf_data_type;

typedef int64_t time64_t;
/*
 * Size of the string in the buffer header that is used to store
 * the serial number and hardware provider information.
 */
#define	SIZEOF_SERIAL_AND_PROVIDER 32

/*
 * There are 2 buffer headers.  Every time a buffer gets used it should
 * change it's header type. This prevents the same bytes being written
 * to the same part of a disk over and over again.
 *
 * There is also an attempt to get the constant parts of this header in
 * one header where the volatile parts are in the other.  This means that
 * if 2 different header types are XORd together as part of a RAID 5
 * volume you can more easily decode what the 2 blocks were. Seems mad
 * but has been proven to be useful when hardware RAID engines go mad.
 *
 * If you change this then be sure to change the PRINT_A_HDR and PRINT_B_HDR
 * macros in bufs.c
 */
struct bufhdr_a {
	char serial_and_provider[SIZEOF_SERIAL_AND_PROVIDER];	/* +32 */
	struct device_id devid;					/* +8 */
	uint32_t hdrchksum;					/* +4 */
	buf_data_type	type;					/* +4 */
	check_t chksum;						/* +8 */
	pid_t did;						/* +4 */
	uint32_t len;						/* +4 */
	ullong_t off;						/* +8 */
	time64_t time;						/* +8 */
};
struct bufhdr_b {
	ullong_t off;						/* +8 */
	time64_t time;						/* +8 */
	uint32_t len;						/* +4 */
	pid_t did;						/* +4 */
	struct device_id devid;					/* +8 */
	uint32_t hdrchksum;					/* +4 */
	buf_data_type	type;					/* +4 */
	check_t chksum;						/* +8 */
	char serial_and_provider[SIZEOF_SERIAL_AND_PROVIDER];	/* +32 */
};
struct shadow_hdr {
	buf_data_type type;
	check_t chksums[1];
};
#define	BUF_TYPE_A 0xAAAAAAAAAAAAAAAAULL
#define	BUF_TYPE_B 0x5555555555555555ULL
struct bufhdr {
	uint64_t start;
	union {
		struct bufhdr_a a;
		struct bufhdr_b b;
		/*
		 * This pad forces sizeof bufhdr.ab to be everything that is
		 * bettween start and end. So that ZERO_OBJ(bufhdr.ab)
		 * clears everything.
		 */
		uint64_t pad[(1 ? 0 : sizeof (struct bufhdr_a) %
			sizeof (uint64_t)) +
			sizeof (struct bufhdr_a) / sizeof (uint64_t)];
	} ab;
	uint64_t end;
};
#define	SIZEOF_BUFHDR sizeof (struct bufhdr)
#define	SIZEOF_BUF 512
union data {
	uchar_t c[(SIZEOF_BUF - SIZEOF_BUFHDR)/sizeof (uchar_t)];
	uint32_t i[(SIZEOF_BUF - SIZEOF_BUFHDR)/sizeof (uint32_t)];
	uint64_t l[(SIZEOF_BUF - SIZEOF_BUFHDR)/sizeof (uint64_t)];
};
struct buf {
	struct bufhdr hdr;
	union data data;
};
extern void init_all_write_bufs(struct aio_str *aio, struct device *devices);
extern struct bufhdr_a conv_bufhdr(struct bufhdr *in);
extern void toggle_bufhdr(uchar_t *buf);
extern struct bufhdr get_bufhdr(uchar_t *buf);
extern struct bufhdr_a get_bufhdr_a(uchar_t *buf);
extern ushort16_t get_bufhdr_hdrchksum(uchar_t *buf);
extern ushort16_t set_hdrchksum(struct bufhdr *hdr);
extern ushort16_t set_buf_hdrchksum(uchar_t *buf);
extern void set_bufhdr_dev(uchar_t *buf, struct device_id devid);
extern void set_bufhdr_off(uchar_t *buf, ullong_t off);
extern ushort16_t check_hdr(struct bufhdr *hdr, ushort16_t hdrchksum);
extern ushort16_t check_bufhdr(uchar_t *buf, ushort16_t hdrchksum);
extern check_t check_aiobuf(struct aio_str *aiop);
extern check_t check_buf(uchar_t *buf, int len, struct error *error);
extern uchar_t *get_read_buf(void);
extern void return_read_buf(uchar_t *buf);
extern uchar_t *get_write_buf(void);
extern int get_write_buf_id(uchar_t *b);
extern void return_write_buf(uchar_t *buf);
extern struct shadow_hdr const * get_shadow_hdr(uchar_t *buf);
extern int is_read_io(struct aio_str *aiop);
extern int is_write_io(struct aio_str *aiop);
extern void decode_header(FILE *out, uchar_t *good, uchar_t *bad);
extern void print_bufhdr_offsets(FILE *out);
extern void init_read_bufs(struct device *devices);
extern uchar_t *get_buf_data(uchar_t *buf);
extern check_t check_bufbody(uchar_t *buf, ulong_t size);
extern char is_executable(uchar_t *buf);
extern void set_bufhdr_all(uchar_t *buf, check_t chksum, uint32_t len,
	struct device_id devid, ullong_t off, buf_data_type type,
	char sequence, time_t time);
extern void set_bufhdr_chksum(uchar_t *buf, ulong_t chksum);
extern void set_bufhdr_len(uchar_t *buf, uint32_t len);
extern void set_bufhdr_type(uchar_t *buf, buf_data_type type);
extern void init_block_str(struct device *device);
extern void init_buf(uchar_t *buf);
extern void save_data_bufs(void);
extern void (*protect_buf)(uchar_t *b);
extern void (*unprotect_buf)(uchar_t *b);
extern time_t get_buf_time(uchar_t *b);
extern pid_t get_buf_did(uchar_t *b);
extern void set_serial_and_provider(void);
extern void get_serial_and_provider(char *buf, int len);
extern const char *get_buf_serial_and_provider(const uchar_t *b);
extern int cmp_serial_and_provider(const char *buf);
extern void print_serial_and_provider(FILE *out);

#ifdef	__cplusplus
}
#endif

#endif /* _DISKOMIZER_BUFS_H */
