/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

#pragma ident	"@(#)daio.c	1.10	09/05/26 SMI"

/*
 *	Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 *	Use is subject to license terms.
 */

/*
 *	diskomizer async io abstraction.
 */
#include <diskomizer/daio.h>
#include <stdlib.h>
#include <ctype.h>
#include <strings.h>
#include <dlfcn.h>
#include <stdio.h>

TNF_DEFINE_RECORD_2(daio_aio_result_t, daio_tnf_aio_result,
	tnf_longlong, daio_return,
	tnf_uint, daio_errno)
TNF_DEFINE_RECORD_2(daio_result_t, daio_tnf_result,
	tnf_longlong, result.daio_return,
	tnf_uint, result.daio_errno)

static struct daio_ops *
do_dlopen(const char *name)
{
	void * handle;
	if ((handle = dlopen(name, RTLD_NOW)) != NULL) {
		struct daio_ops *ops;

		if ((ops = dlsym(handle, DAIO_OPS_STRING)) == NULL) {
			dlclose(handle);
		} else {
			return (ops);
		}
	}
	return (NULL);
}
static void
lower(char *x)
{
	while (*x != NULL) {
		*x = tolower(*x);
		x++;
	}
}
static const char libdaio_name[] = "libdaio_%s.so";

static struct daio_ops *
open_daio(char *libname, const char *name)
{
	sprintf(libname, libdaio_name, name);
	lower(libname);
	return (do_dlopen(libname));
}

struct daio_ops *
daio_choose_ops(const char *name)
{
	struct daio_ops *ops = NULL;
	char *x = (char *)name;

	if ((ops = do_dlopen(name)) != NULL) {
		return (ops);
	} else {
		char *full_name;
		/*
		 * Have to allocate enough space for the string in 'name'
		 * and the string "libdaio_%s.so" and the trailing NULL.
		 */
		if ((full_name = malloc(strlen(name) +
		    strlen(libdaio_name) + 1)) != NULL) {
			x = full_name;
			ops = open_daio(full_name, name);
			if (ops == NULL) {
				char *tname;

				tname = strdup(name);
				if (tname != NULL) {
					char *space;
					char again = 0;

					space = strchr(tname, ' ');
					if (space != NULL) {
						*space = NULL;
						again = 1;
					}
					space = strchr(tname, '\t');
					if (space != NULL) {
						*space = NULL;
						again = 1;
					}
					if (again) {
						ops = open_daio(full_name,
						    tname);
					}
					free(tname);
				}
			}
		}
	}
	if (ops == NULL) {
		fprintf(stderr, "Err %s: %s\n", x, dlerror());
	}
	if (name != x) {
		free(x);
	}
	return (ops);
}
