/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

#pragma ident	"@(#)daio_dev.c	1.6	09/05/26 SMI"

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#include <diskomizer/daio.h>
#include <diskomizer/checker.h>
#include "findap.h"
#include <sys/asynch.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <diskomizer/assert.h>

#pragma weak directio

struct daio_devs {
	int max_fd;
	char **paths;
};

static struct daio_devs dev_names;

int
daio_dev_open(const char *path, int flags, mode_t mode)
{
	int fd;
	fd = open(path, flags, mode);
	if (fd < 0) {
		return (fd);
	}

	if ((fd + 1) > dev_names.max_fd) {
		char **x;
		int newlen;

		for (newlen = dev_names.max_fd == 0 ? 4 : dev_names.max_fd * 2;
		    newlen <= fd; newlen *= 2)
			/* EMPTY */;

		x = realloc(dev_names.paths, newlen * sizeof (char *));
		if (x == NULL) {
			close(fd);
			return (-1);
		}
		dev_names.paths = x;
		memset(&dev_names.paths[dev_names.max_fd], 0,
		    sizeof (char *) * (newlen - dev_names.max_fd));
		dev_names.max_fd = newlen;
	}
	dev_names.paths[fd] = strdup(path);
	if (dev_names.paths[fd] == NULL) {
		close(fd);
		fd = -1;
	}
	return (fd);
}

void
daio_dev_dd(int pri, int fd, size_t len, size_t iolen, off_t off)
{
	off64_t x = off / (off64_t)iolen;

	plog(pri, "use \""
	    "dd if=%s bs=%d iseek=%lld count=1\" to read the "
	    "block\n",
	    dev_names.paths[fd], len, (long long)x);
}

const char *
daio_dev_path(int fd)
{
	return (dev_names.paths[fd]);
}

int
daio_dev_close(int fd)
{
	free(dev_names.paths[fd]);
	dev_names.paths[fd] = NULL;
	return (close(fd));
}

int
daio_dev_directio(int fd, int advice)
{
	if (directio == NULL) {
		return (-1);
	} else {
		return (directio(fd, advice));
	}
}
