/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

#pragma ident	"@(#)daio_pread.c	1.14	09/05/26 SMI"

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#include <diskomizer/daio.h>
#include <diskomizer/daio_dev.h>
#include "findap.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include "async.h"
#include "daio_async.h"

static int daio_prw_write(int fildes, uchar_t *bufp, int bufs, off_t
		offset, daio_result_t *resultp, struct daio_id *id);
static int daio_prw_read(int fildes, uchar_t *bufp, int bufs, off_t
		offset, daio_result_t *resultp, struct daio_id *id);

struct daio_ops DAIO_OPS = {
	daio_async_init,
	daio_async_init_checker,
	daio_async_get_checker,
	findap,
	(daio_open_t)daio_dev_open,
	daio_dev_close,
	unlink,
	stat64,
	fstat64,
	ioctl,
	ftruncate64,
	daio_dev_directio,
	read_disko_vtoc,
	daio_async_pwrite,
	daio_async_pread,
	daio_prw_write,
	daio_prw_read,
	daio_async_status,
	daio_async_start_time,
	daio_async_end_time,
	daio_async_wait,
	daio_async_cancel,
	daio_async_fini,
	daio_dev_dd
};

static int
daio_prw_read(int fildes, uchar_t *bufp, int bufs, off_t offset,
		daio_result_t *resultp, struct daio_id *id)
{
	return (daio_async_rw(fildes, bufp, bufs, offset, resultp, id,
	    pread, IS_READ));
}
static int
daio_prw_write(int fildes, uchar_t *bufp, int bufs, off_t offset,
		daio_result_t *resultp, struct daio_id *id)
{
	return (daio_async_rw(fildes, bufp, bufs, offset, resultp, id,
	    (ssize_t (*)(int fildes, void  *buf,
	    size_t nbyte, off_t offset)) pwrite, IS_WRITE));
}
