/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)errors.h	1.11	09/05/26 SMI"

#ifndef _ERRORS_H
#define	_ERRORS_H

#ifdef	__cplusplus
extern "C" {
#endif

#include <libintl.h>
#include <errno.h>

#define	SHM_INIT_ERROR(A, B) \
	pperror(gettext("shm_%s_init(%#lx) failed, %s:%d"), \
	shm_ops->name(A), B, __FILE__, __LINE__)
#define	ATTACH_ERROR(A) plog(LOG_WARNING, \
	gettext("shm_%s_attach failed, %s:%d %s"), \
	shm_ops->name(A), __FILE__, __LINE__, strerror(errno))
#define	OPEN_ERROR(A, B) pperror(gettext("Unable to open(%s, %s)"), A, #B)
#define	FOPEN_ERROR(A, B) pperror(gettext("Unable to fopen(%s, %s)"), A, B)
#define	STAT_ERROR(A) pperror(gettext("Unable to stat(%s)"), A)
#define	FSTAT_ERROR(A, B) pperror(gettext("Unable to fstat(%d (%s))"), A, B)
#define	FSYNC_ERROR(A, B) pperror(gettext("Unable to fsync(%d (%s))"), A, B)
#define	CLOSE_ERROR(A, B) pperror(gettext("Unable to close(%d (%s))"), A, B)
#define	CALLOC_ERROR(A, B) pperror(gettext("calloc(%#lx, %#lx)"), A, B)
#define	MALLOC_ERROR(A) pperror(gettext("malloc(%ld)"), A)
#define	STRDUP_ERROR(A) pperror(gettext("strdup(%s)"), A)
#define	REALLOC_ERROR(A, B) pperror(gettext("realloc(%lx, %ld)"), (ulong_t)A, \
	(ulong_t)B)
#define	AIOREAD_ERROR(A, S, B, C, D, E, F) \
	pperror(gettext("aioread(%d (%s), %#lX, %#X, %#llX, %s, %#lX)"), \
	A, S, (ulong_t)B, C, D, #E, (ulong_t)F)
#define	AIOWRITE_ERROR(A, S,  B, C, D, E, F) \
	pperror(gettext("aiowrite(%d (%s), %#lX, %#X, %#llX, %s, %#lX)"), \
	A, S, (ulong_t)B, C, D, #E, F)
#define	AIOWAIT_ERROR(T) pperror(gettext("aiowait(%ld.%5.5ld)"), \
	T.tv_sec, T.tv_usec)
#define	AIOCANCEL_ERROR(A) pperror(gettext("aiocancel(%d,%d (%s)"), \
	DAIO_RETURN((A)->aio_res), DAIO_ERROR((A)->aio_res), \
	(A)->fd->name)
#define	FORK_ERROR(A) pperror(gettext("fork%s"), A)
#define	PREAD_ERROR(A, B, C, D, E) \
	pperror(gettext("pread(%d (%s), %#lx, %d, %#llx)"), \
	A, B, C, D, (long long)E)
#define	PWRITE_ERROR(A, B, C, D, E) \
	pperror(gettext("pwrite(%d (%s), %#lx, %d, %#lx)"), \
	A, B, C, D, E)
#define	MADVISE_ERROR(A, B, C, D) \
	pperror(gettext("madvise(%#lx, %#x, %#x (%s))"), \
	A, B, C, D);

#ifdef	__cplusplus
}
#endif

#endif	/* _ERRORS_H */
