/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)libstack_trace.c	1.13	09/05/26 SMI"

/*
 * print a SPARC user land stack from a dynamic program.
 *
 *	Chris Gerhard - Computer Systems CTE - Sun Microsystems
 *	Chris.Gerhard@uk.sun.com
 */
#ifndef _LARGEFILE64_SOURCE
#define	_LARGEFILE64_SOURCE
#endif

#include <sys/stack.h>
#include <dlfcn.h>
#include <ucontext.h>
#include <stdio.h>
#include <diskomizer/stack_trace.h>
#include <diskomizer/assert.h>
#ifdef __sparc
#include <demangle.h>
#include "sparc_asm.h"
#define	FRAME_SP 6
#ifndef STACK_BIAS
#define	STACK_BIAS 0
#endif

struct frame {
	void *l[8];
	void *i[8];
};
void
ftraceback(int (*func)(void *arg, const char *fmt, ...),
	void *out, ucontext_t *ucp)
{
	Dl_info dli;
	struct frame *f;
	const char *x;
	ulong_t diff;
	char buf[1024];
	ulong_t stack;
#define	pc  ucp->uc_mcontext.gregs[REG_PC]
	flush_windows();
	if (dladdr((void *)ucp->uc_mcontext.gregs[REG_PC], &dli) <= 0) {
		func(out,
		    "pc %#x %s:%#x "
		    "(%#x, %#x, %#x, %#x, %#x, %#x)\n",
		    pc, "unknown", pc,
		    ucp->uc_mcontext.gregs[REG_O0],
		    ucp->uc_mcontext.gregs[REG_O1],
		    ucp->uc_mcontext.gregs[REG_O2],
		    ucp->uc_mcontext.gregs[REG_O3],
		    ucp->uc_mcontext.gregs[REG_O4],
		    ucp->uc_mcontext.gregs[REG_O5]);
	} else {
		diff = (ulong_t)pc - (ulong_t)dli.dli_saddr;

		if (cplus_demangle(dli.dli_sname, buf, sizeof (buf)) !=
		    DEMANGLE_ESPACE)
			x = &buf[0];
		else
			x = dli.dli_sname;

		if (diff)
			func(out,
			    "pc %#lx %s:%s+%#lx "
			    "(%#lx, %#lx, %#lx, %#lx, %#lx, %#lx)\n",
			    ucp->uc_mcontext.gregs[REG_PC],
			    dli.dli_fname, x, diff,
			    ucp->uc_mcontext.gregs[REG_O0],
			    ucp->uc_mcontext.gregs[REG_O1],
			    ucp->uc_mcontext.gregs[REG_O2],
			    ucp->uc_mcontext.gregs[REG_O3],
			    ucp->uc_mcontext.gregs[REG_O4],
			    ucp->uc_mcontext.gregs[REG_O5]);
		else
			func(out,
			    "pc %#lx %s:%s "
			    "(%#lx, %#lx, %#lx, %#lx, %#lx, %#lx)\n",
			    ucp->uc_mcontext.gregs[REG_PC],
			    dli.dli_fname, x,
			    ucp->uc_mcontext.gregs[REG_O0],
			    ucp->uc_mcontext.gregs[REG_O1],
			    ucp->uc_mcontext.gregs[REG_O2],
			    ucp->uc_mcontext.gregs[REG_O3],
			    ucp->uc_mcontext.gregs[REG_O4],
			    ucp->uc_mcontext.gregs[REG_O5]);
	}
#undef pc
#define	pc f->i[7]
	stack = (ulong_t)ucp->uc_mcontext.gregs[REG_SP];
	f = (struct frame *)(stack + STACK_BIAS);

	for (stack = (ulong_t)ucp->uc_mcontext.gregs[REG_SP]; stack != 0 &&
	    pc != 0; stack  = (ulong_t)f->i[FRAME_SP]) {
		f = (struct frame *)(stack + STACK_BIAS);
		if (dladdr(pc, &dli) <= 0) {
			func(out,
			    "pc %#lx %s:%#lx "
			    "(%#lx, %#lx, %#lx, %#lx, %#lx, %#lx)\n",
			    pc, "unknown", pc, diff,
			    f->i[0], f->i[1], f->i[2], f->i[3],
			    f->i[4], f->i[5]);
			continue;
		}
		diff = (ulong_t)pc - (ulong_t)dli.dli_saddr;

		if (cplus_demangle(dli.dli_sname, buf, sizeof (buf)) !=
		    DEMANGLE_ESPACE)
			x = &buf[0];
		else
			x = dli.dli_sname;

		if (diff)
			func(out,
			    "pc %#lx %s:%s+%#lx "
			    "(%#lx, %#lx, %#lx, %#lx, %#lx, %#lx)\n",
			    pc, dli.dli_fname, x, diff,
			    f->i[0], f->i[1], f->i[2], f->i[3],
			    f->i[4], f->i[5]);
		else
			func(out,
			    "pc %#lx %s:%s "
			    "(%#lx, %#lx, %#lx, %#lx, %#lx, %#lx)\n",
			    pc, dli.dli_fname, x,
			    f->i[0], f->i[1], f->i[2], f->i[3],
			    f->i[4], f->i[5]);
	}
#undef pc
}
#else /* Intel */
#include <strings.h>

/*
 * A not totally random guess at the length of the call instruction
 * used. This needs to be subtracted from the return address to
 * give an accurate picture of whence we came.
 */

#define	CALL_LEN	5UL
#define	ARGC_MAX	6

void
ftraceback(int (*func)(void *arg, const char *fmt, ...),
    void *out, ucontext_t *ucp)
{
	Dl_info dli;
	ulong_t **ebp, **oebp;
	ulong_t *args, *ret;
	ulong_t nargs, pc, diff;
	const char *x;
#ifdef DO_DEMANGLE
	char buf[128];
#endif

	ebp = (ulong_t **)(ucp->uc_mcontext.gregs[6]);
	pc = ((ulong_t)(ucp->uc_mcontext.gregs[14])-CALL_LEN);

	while (*ebp) {
		char tmp[1024];
		tmp[0] = NULL;
		oebp = (ulong_t **)*ebp;
		ret = ((ulong_t *)ebp + 1);

		args = ret + 1;
		nargs = ((ulong_t *)oebp) - args;

		if (dladdr((void *)pc, &dli) <= 0) {
			snprintf(tmp, sizeof (tmp), "pc %#x %s:%#x ",
			    pc, "unknown", pc);
		} else {
			diff = (ulong_t)pc - (ulong_t)dli.dli_saddr;

			/*
			 * A bug in the libdemangle.so on Intel
			 * means we currently don't handle mangled
			 * c++ object names.
			 */
#ifdef DO_DEMANGLE
			if (cplus_demangle(dli.dli_sname, buf,
			    sizeof (buf)) != *DEMANGLE_ESPACE) {
				x = &buf[0];
			} else {
				x = dli.dli_sname;
			}
#else
			x = dli.dli_sname;
#endif

			if (diff) {
				snprintf(tmp, sizeof (tmp),
				    "pc %#lx %s:%s+%#lx ",
				    pc, dli.dli_fname, x, diff);
			} else {
				snprintf(tmp, sizeof (tmp), "pc %#lx %s:%s ",
				    pc, dli.dli_fname, x);
			}
		}

		if (!nargs) {
			int l = strlen(tmp);
			snprintf(&tmp[l], sizeof (tmp) - l, "%s", "()");
		} else {
			int i, l;
			l = strlen(tmp);
			snprintf(&tmp[l], sizeof (tmp) - l, "(");
			for (i = 0; i < ARGC_MAX && nargs; i++) {
				l = strlen(tmp);
				if (snprintf(&tmp[l], sizeof (tmp) - l,
				    "%#x%s", *args, --nargs ? ", ": ")") >
				    sizeof (tmp) - l)
						break;
				args += 1;
			}
		}
		func(out, "%s\n", tmp);

		ebp = oebp;
		pc = (*ret)-CALL_LEN;
	}
}
#endif /* Intel */
void
traceback(FILE *out, ucontext_t *ucp)
{
	ftraceback((int (*)(void *, const char *, ...))fprintf, (void *)out,
	    ucp);
	(void) fflush(out);
}
#ifdef  __cplusplus
extern "C" {
#endif
void
fprint_stack_trace(int (*out)(void *, const char *, ...),
	void *arg)
{
	ucontext_t context;

#ifdef __sparc
	flush_windows();
#endif
	if (getcontext(&context) == -1)
		return;

	ftraceback(out, arg, &context);
}
void
print_stack_trace(FILE *out)
{
	fprint_stack_trace((int (*)(void *, const char *, ...))fprintf,
	    (void *)out);
}
#ifdef  __cplusplus
}
#endif
