/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)limit.c	1.7	09/05/26 SMI"

#include <sys/resource.h>
#include "limit.h"
#include "args.h"
#include <diskomizer/log.h>

void
set_limits(void)
{
	struct rlimit rl;

	if (getrlimit(RLIMIT_NOFILE, &rl) == -1) {
		pperror("getrlimit(RLIMIT_NOFILE, &rl)");
		return;
	}
	rl.rlim_cur = rl.rlim_max;
	if (setrlimit(RLIMIT_NOFILE, &rl) == -1) {
		pperror("setrlimit(RLIMIT_NOFILE, max = %ld, cur %ld)",
		    rl.rlim_max, rl.rlim_cur);
	}
	if (!opts.debug_allow_coredump) {
		if (getrlimit(RLIMIT_CORE, &rl) == -1) {
			pperror("getrlimit(RLIMIT_CORE, &rl)");
			return;
		}
		rl.rlim_cur = 0;
		if (setrlimit(RLIMIT_CORE, &rl) == -1) {
			pperror("setrlimit(RLIMIT_CORE, max = %ld, cur %ld)",
			    rl.rlim_max, rl.rlim_cur);
		}
	}
}
