/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)list_ops.c	1.6	09/05/26 SMI"

#include "diskomizer64mpism.h"
#include "list_ops.h"

void
add_to_aio_list(struct aioqtop *list, struct aio_str *aiop)
{
	aiop->next = NULL;
	aiop->prev = list->tail;
	if (list->tail != NULL) {
		(list->tail)->next = aiop;
	} else {
		/* if tail is NULL the queue must be empty */
		assert(list->head == NULL);
		list->head = aiop;
	}
	list->tail = aiop;
}

void
remove_from_aio_list(struct aioqtop *list, struct aio_str *aiop)
{
	if (aiop->prev == NULL) {
		assert(aiop == list->head);
		list->head = aiop->next;
	} else {
		aiop->prev->next = aiop->next;
	}
	if (aiop->next == NULL) {
		assert(aiop == list->tail);
		list->tail = aiop->prev;
	} else {
		aiop->next->prev = aiop->prev;
	}
	aiop->prev = aiop->next = NULL;
}

int
is_aio_on_list(struct aioqtop *list)
{
	return (list->head == NULL ? 0 : 1);
}

struct aio_str *
pop_from_aio_list(struct aioqtop *list)
{
	struct aio_str *item;

	if ((item = list->head) != NULL) {
		remove_from_aio_list(list, item);
	}
	return (item);
}
