/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)prompt.c	1.7	09/05/26 SMI"

#include <diskomizer/log.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include "args.h"
#include "prompt.h"
#include "errors.h"


static void
remove_newline(char *str)
{
	int len;
	len = strlen(str);

	if (str[--len] == '\n')
		str[len] = NULL;
}
static void
append(char **str, char *line)
{
	int len;
	char *tmp;
	if (*str == NULL) {
		len = strlen(line) + 1;
	} else {
		len = strlen(line) + strlen(*str) + 2;
	}

	if ((tmp = (char *)realloc(*str, len)) == NULL) {
		REALLOC_ERROR(*str, len);
		exit(1);
	}
	if (*str != NULL) {
		(void) strcat(tmp, " ");
	} else {
		*tmp = NULL;
	}
	(void) strcat(tmp, line);
	remove_newline(tmp);
	*str = tmp;
}
static char *
getline(FILE *tty, const char *prompt)
{
	static char buf[1024];
	char *x;
	FILE *ttyin;
	(void) fprintf(tty, prompt);
	(void) fflush(tty);
	if ((ttyin = fopen("/dev/tty", "r")) == NULL)
		return (0);

	x = fgets(&buf[0], sizeof (buf), ttyin);
	(void) fclose(ttyin);

	return (x);
}

int
prompt(void)
{
	FILE *tty;
	char *res = NULL;
	char *line;

	if ((tty = fopen("/dev/tty", "w")) == NULL)
		return (0);
	(void) fprintf(tty,
	    "Enter devices to test, one device per line, end with a ^D\n");
	(void) fflush(tty);
	while ((line = getline(tty, "Device: ")) != NULL) {
		append(&res, line);
	}
	opts.device = res;
	if ((line = getline(tty,
	    "Would you like to save this config Y/N?:")) != NULL) {
		if (*line == 'Y' || *line == 'y') {
			do {
				line = getline(tty,
				    "Enter file name to save into:");

				if (line == NULL)
					exit(1);

				remove_newline(line);
			} while (!save_options(line));
		}
	}

	(void) fclose(tty);
	return (1);
}

#ifdef MAIN_PROMPT
void
main(int argc, char **argv)
{

	prompt();
	printf("opts.device = %s\n", opts.device);
}
#endif
