/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)shm_ops.c	1.15	09/05/26 SMI"

#include <sys/types.h>
#include <strings.h>
#include <assert.h>
#include <stdlib.h>

#include <diskomizer/log.h>
#include "shm_ops.h"
#include "args.h"
#define	ZERO_OBJ(X) (void) memset(&X, NULL, sizeof (X))

static struct shm_ops *all_shm_ops[] =
	{&shm_ism_ops, &shm_shm_ops, &shm_mmap_ops, &shm_mmap_no_frag_ops,
		&shm_bestshm_ops, &shm_best_ops, NULL};

struct shm_ops *
choose_shm_ops(const char *key)
{
	int i;
	for (i = 0; all_shm_ops[i] != NULL; i++) {
		if (strcasecmp(key, all_shm_ops[i]->name(NULL)) == 0)
			return (all_shm_ops[i]);
	}
	return (&shm_best_ops);
}
#ifndef _LP64
static void
check_4_overflow(long a, long b)
{
	long long big;
	ulong_t little;

	big = a;
	big *= b;

	/*LINTED*/
	little = big;

	if (big != (long long) little) {
		pprintf("Unable to allocate %#llx bytes in a %ld (%#lx) bit\n",
		    big, sizeof (long) * 8, sizeof (long) * 8);
		pprintf("address space, exiting\n");
		exit(1);
	}
}
#else
#define	check_4_overflow(A, B)
#endif
void *
alloc_mem(long a, long b)
{
	void *handle, *addr;
	shm_flags flags;
	ZERO_OBJ(flags);

	flags.execute = opts.obscure_execute;
	assert(a);
	assert(b);
	check_4_overflow(a, b);
	if ((handle = shm_ops->init(a, b, flags)) == NULL) {
		pperror("Init using %s(%#lx) failed", shm_ops->longname(NULL),
		    a * b);
		exit(1);
	}
	plog(LOG_DEBUG, "alloc_mem handle %#lx\n", (ulong_t)handle);
	if ((addr = shm_ops->attach(handle)) == NULL) {
		pperror("Attach using %s(%#lx) failed",
		    shm_ops->longname(addr), a * b);
		exit(1);
	}
	return (addr);
}
