/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)shm_ops.h	1.12	09/05/26 SMI"

#ifndef _DISKOMIZER_SHM_OPS_H
#define	_DISKOMIZER_SHM_OPS_H

#ifdef	__cplusplus
extern "C" {
#endif

#include <sys/types.h>
struct shm_flags {
	unsigned execute : 1;
	unsigned write : 1;
	unsigned allow_detach : 1;
	unsigned always_detach : 1;
	unsigned keep_shm : 1;
	unsigned leave_attached: 1;
	unsigned private : 26;
};
typedef struct shm_flags shm_flags;
typedef enum {
	SHM_DETACH_ERR,
	SHM_DETACH_OK,
	SHM_DETACH_DONE
} shm_det_status;
struct shm_ops {
	const char *(*name)(void *handle);
	const char *(*longname)(void *handle);
	/* init returns an opaque handle */
	void * (*init)(long a, long b, shm_flags flags);
	void * (*attach)(void *handle);
	shm_det_status (*detach)(void *handle);
	void (*destroy)(void *handle);
	ulong_t (*max_size)(void);
	int (*is_short_of_mem)(void);
	/* Called when all allocations have been done */
	void (*complete)(void *handle);
	void (*fini)(void);
/* 	int (*garbage_collect)(void * addr, size_t size); */
};
extern struct shm_ops shm_shm_ops;
extern struct shm_ops shm_ism_ops;
extern struct shm_ops shm_bestshm_ops;
extern struct shm_ops shm_mmap_ops;
extern struct shm_ops shm_mmap_no_frag_ops;
extern struct shm_ops shm_best_ops;
extern struct shm_ops *shm_ops;
extern struct shm_ops *choose_shm_ops(const char *key);
extern void * alloc_mem(long a, long b);
#define	ROUND_UP(A, B) ((A) % (B) == 0 ? (A) : ((A) + (B) - ((A) % (B))))

#ifdef	__cplusplus
}
#endif

#endif	/* _DISKOMIZER_SHM_OPS_H */
