#!/bin/ksh -p
#
# CDDL HEADER START
#
# The contents of this file are subject to the terms of the
# Common Development and Distribution License (the "License").
# You may not use this file except in compliance with the License.
#
# You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
# or http://www.opensolaris.org/os/licensing.
# See the License for the specific language governing permissions
# and limitations under the License.
#
# When distributing Covered Code, include this CDDL HEADER in each
# file and include the License file at usr/src/OPENSOLARIS.LICENSE.
# If applicable, add the following below this CDDL HEADER, with the
# fields enclosed by brackets "[]" replaced with your own identifying
# information: Portions Copyright [yyyy] [name of copyright owner]
#
# CDDL HEADER END
#

#
# Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
# Use is subject to license terms.
#
#ident	"@(#)load_test.sh	1.5	09/05/26 SMI"

# Simple script to kick off diskomizer and run iostat in the background
# total up the iostat and report. Used to verify that diskomizer can reach the
# required performance goals.
#
# Since the goal is to get the devices to their throttle, the statistic
# looked at is the dirves wait queue as this is only used after reaching
# the throttle.

ionawk()
{
nawk "/^${1:-ssd}[0-9]/"' {
	if ($2 == 0 && $3 == 0 && $4 == 0 && $5 == 0 && $6 == 0 && $7 == 0 ) {
		next;
	} else {
		wait += $6;
		if ($6 > wmax) { wmax = $6} ; if ($6 < wmin) wmin = $6 ;
		count++
	}
}
function reset_wait(x)
{
	wmin = 10000;
	wmax = wait = count = 0;
}
function totalup(x)
{
	if (x <= 1){
		reset_wait(1);
		return;
	}
	if (count != 0)
		printf("%6.6g, %6.6g, %6.6g\n", wmin,wait/count,wmax);
	if (tmax < wmax)
		tmax = wmax;
	if (tmin > wmin)
		tmin = wmin;
	twait += wait;
	tcount += count;
	reset_wait(1);
}
/^device/ {
	totalup(y);
	y++;
}
BEGIN { tmin = 10000 ; tmax = 0; y=0;
		printf("%6.6s, %6.6s, %6.6s\n", "Min", "Mean", "Max")}
END {
	totalup(y);
	printf("Totals\n----------------------\n%6.6g, %6.6g, %6.6g\n",
		tmin, twait/tcount, tmax);
	if (twait/tcount > 1) {
		printf("Looks o.k.\n");
	}
}'
}
usage()
{
	echo "Usage:" >&2
	echo "$0 [-x executable][ -i interval in minutes ][ -t time to run specified in the number of intervals ] option_file" >&2
	exit 1
}
killit()
{
	if [[ ${!:-0} -ne 0 ]]
	then
		echo "Killing $!"
		kill $!
	fi
}
trap usage EXIT
typeset -i time
while getopts "i:x:t:" c
do
	case $c in
	x) executable=$OPTARG ;;
	t) time=$OPTARG ;;
	i) interval=$OPTARG ;;
	\?) exit ;;
	esac
done
shift $(( $OPTIND - 1))
while [[ $# -gt 0 ]]
do
	opts="${opts:+${opts} }-f $1"
	shift
done
[[ -x ${executable:-/opt/CTEdiskomizer/bin/diskomizer64mpism} ]] || exit
trap "killit" EXIT
echo "running for $((${interval:-60} * 60 * ${time:-24})) seconds."
echo "reporting every $((${interval:-60} * 60)) seconds."
set -x
${executable:-/opt/CTEdiskomizer/bin/diskomizer64mpism} ${opts:-} \
	-f background -o background=0 \
	-o SECONDS_TO_RUN=$((${interval:-60} * 60 * ${time:-24})) &
set +x
echo "diskomizer pid $!"

# Since we wish to run for 24 hours we have to set the count to one more than
# the tester requests to cover the first report done by iostat.
iostat -x $(( ${interval:-60} * 60)) $(( ${time:-24} + 1)) |  ionawk
wait
trap "" EXIT
