#!/bin/ksh
#
# CDDL HEADER START
#
# The contents of this file are subject to the terms of the
# Common Development and Distribution License (the "License").
# You may not use this file except in compliance with the License.
#
# You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
# or http://www.opensolaris.org/os/licensing.
# See the License for the specific language governing permissions
# and limitations under the License.
#
# When distributing Covered Code, include this CDDL HEADER in each
# file and include the License file at usr/src/OPENSOLARIS.LICENSE.
# If applicable, add the following below this CDDL HEADER, with the
# fields enclosed by brackets "[]" replaced with your own identifying
# information: Portions Copyright [yyyy] [name of copyright owner]
#
# CDDL HEADER END
#

#
# Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
# Use is subject to license terms.
#
#ident	"@(#)option_test.sh	1.5	09/05/26 SMI"

#	Script to test the option handling of diskomizer.
#
OPT_FILE=/tmp/disko_test.$$
OPT_FILE2=/tmp/disko_test_2.$$
save_or_remove_files()
{
	echo "save or remove $@"
	while [ $# -gt 0 ]
	do
		save_or_remove $1
		shift
	done
}
save_or_remove()
{
	if ! [ -f $1 ]
	then
		return
	fi
	typeset file spam
	if [ "$(yesno would you like to save file $1)" = "y" ]
	then
		while :
		do
			echo "Enter filename: \c"
			read file spam
			if [ "$(yesno "filename is \"$file\" is this correct")" = "y" ]
			then
				if cp -i $1 $file
				then
					break
				fi
				echo "Unable to copy to $file"
			fi
		done
	fi
	rm -f $1
}
yesno()
{
	typeset -l a b
	echo "$@ [y/n]? \c" >&2
	while read a b
	do
		if [ "$a" = "y" -o "$a" = "n" ]
		then
			echo $a
			return 
		fi
		echo "$a is not a valid respnse" >&2
		echo "enter y or n: \c" >&2
	done
	echo "end of file seen, exiting test...." >&2
	exit 1
}
parse_error1()
{
	typeset ERROR
	ERROR=WTHREADS=100x
	cat > ${OPT_FILE} << EOF
STDERR=/dev/null
STDOUT=/dev/null
DEVICE=/dev/null
#
# \\

${ERROR}
EOF
	parse_error "${ERROR}" "${OPT_FILE}"
}
findline()
{
	line=$(fgrep -n "$1" $2 | nawk -F: '{ print $1}')
	if [ ${line} != "" ]
	then
		echo ${line}
	else
		echo 0
	fi
}
parse_error2()
{
	typeset ERROR
	typeset -i c
	ERROR=WTHREADS=100kb
	cat > ${OPT_FILE} << EOF
STDERR=/dev/null
STDOUT=/dev/null
DEVICE=/dev/null
#
# \\

$ERROR
RTHREADS=4

EOF
	parse_error "$ERROR" "${OPT_FILE}"
	return $?
}
parse_error3()
{
	typeset ERROR
	typeset -i c
	ERROR='	* 20'
	cat > ${OPT_FILE} << EOF
STDERR=/dev/null
STDOUT=/dev/null
DEVICE=/dev/null
#
WTHREADS= 10 * \\
$ERROR

EOF
	parse_error "$ERROR" "${OPT_FILE}"
	return $?
}
run_diskomizer_nomore()
{
	typeset -i status
	typeset OPT_FILE LINE TEST c command
	while getopts f:d:o: c
	do
		case $c in
		f)	OPT_FILE=$OPTARG 
			echo OPT_FILE=${OPT_FILE};;
		o)	OPT_LINE=$OPTARG ;;
		d)	DUMP_FILE=$OPTARG 
			echo DUMP_FILE=${DUMP_FILE};;
		\?) echo "Unkownn flar $c"
			exit 1 ;;
		esac
	done
	shift `expr $OPTIND - 1`

	TEST="$@"
	shift
	echo ${TEST}
	cat -n ${OPT_FILE}
	command=diskomizer64mpism${OPT_FILE:+ -f ${OPT_FILE}}${OPT_LINE:+ -o ${OPT_LINE}}${DUMP_FILE:+ -d ${DUMP_FILE}}
	echo "\nRunning \"${command}\":-\n"
	${command}
	status=$?
	echo "\nDiskomizer exited with status ${status}"
	return ${status}
}
run_diskomizer()
{
	run_diskomizer_nomore ${1+"$@"} 2>&1 |more
	trap "save_or_remove_files ${OPT_FILE} ${OPT_FILE2}" EXIT
}
parse_error()
{
	typeset ERROR OPT_FILE
	ERROR=$1
	shift
	OPT_FILE=$1
	shift
	c=$(findline "$ERROR" "${OPT_FILE}")
	run_diskomizer -f ${OPT_FILE} \
		"\ndoing parse error test  on line $c of file ${OPT_FILE}\n\n"
	if [ "$(yesno "\ndid diskomizer spot the error on the line \"${ERROR}\"" )" = "n" ]
	then
		exit 1
	fi
	if [ "$(yesno "\ndid diskomizer get the line number correct")" = "n" ]
	then
		exit 1
	fi
}

check_option()
{
	file=$1
	shift
	option=$1
	shift
	value=$1
	shift
	foo="$(sed -e 's/\#.*$//g' < ${file} |\
		nawk -F= -v x=${option} '{ if ( x == $1 ) print $2 }' )"

	if [ "${foo}" -ne "$value" ]
	then
		echo "$option has wrong value was $foo should be $value"
		exit 1
	else
		echo "No errors found in option file. Maths agree"
	fi
}
parse_ok()
{
typeset -i WT WRT RT
	WT=40
	WRT=50


cat > ${OPT_FILE} << EOF
STDERR=/dev/null
STDOUT=/dev/null
DEVICE=/dev/null
#
WTHREADS=$WT
WRTHREADS=$WRT
RTHREADS= 10 * \\
	( \${WTHREADS} + \${WRTHREADS} )
SECONDS_TO_RUN=60*60*12
EOF
	run_diskomizer -f ${OPT_FILE} -d ${OPT_FILE2} \
		"\nTesting a good option file.\n\nOption file:\n\n"
	
	let y=${WT}+${WRT}
	let x=10*y
	check_option ${OPT_FILE2} RTHREADS $x
}


trap "save_or_remove_files ${OPT_FILE} ${OPT_FILE2}" EXIT

parse_error1
parse_error2
parse_error3
parse_ok
rm -f ${OPT_FILE} ${OPT_FILE2}
exit 0
