/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)time.c	1.8	09/05/26 SMI"

#include "time.h"
/*
 *  This check finds bugs of the sort 4126459 where usec is greater than
 *  1, 000, 000 which makes no sense.
 */
int
my_gettimeofday(struct timeval *tp, void *x)
{
	int stat;
	int i;

	for (i = 0; i <= opts.max_gettimeofday_retries; i++) {
		stat = gettimeofday(tp, x);
		if (stat != -1 && tp->tv_usec > 1000000) {
			pfprintf(stderr, "gettimeofday: bogus usec %ld\n",
			    tp->tv_usec);
			continue;
		}
		return (stat);
	}
	exit(1);
	/*NOTREACHED*/
}
char *
alloc_time_str_fmt(time_t tv, char *fmt)
{
	struct tm *tm;
	char *str = NULL;
	size_t size = 1024;

	tm = localtime(&tv);

	while ((str = realloc(str, size)) != NULL) {
		if (tm == NULL) {
			snprintf(str, size, "bad time");
			break;
		}
		if (strftime(str, size, fmt, tm) != 0)
			break;
		size += 1024;
	}
	return (str);
}
char *
alloc_time_str(time_t tv)
{
	return (alloc_time_str_fmt(tv, NULL));
}
char *
alloc_time_now_fmt(char *fmt)
{
	struct timeval tv;
	while (my_gettimeofday(&tv, NULL) == -1)
		pperror("gettimeofday");

	return (alloc_time_str_fmt(tv.tv_sec, fmt));
}
char *
alloc_time_now(void)
{
	return (alloc_time_now_fmt(NULL));
}
