/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"@(#)uadmin.c	1.7	09/05/26 SMI"

#include "args.h"
#include <diskomizer/log.h>
#include "diskomizer64mpism.h"
#include <sys/uadmin.h>
#include <stdio.h>
#include <errno.h>

struct ua {
	int i;
	char *s;
};

#ifndef A_DUMP
#define	A_DUMP 0x5
#endif

#define	U(A) { A, #A }
static struct ua uadmin_cmd[] = {
	U(A_SHUTDOWN),
	U(A_REBOOT),
	U(A_DUMP),
	U(A_REMOUNT),
	U(A_FREEZE),

};

static struct ua uadmin_fcn[] = {
	U(AD_HALT),
	U(AD_POWEROFF),
	U(AD_BOOT),
	U(AD_IBOOT),
	U(AD_COMPRESS),
	U(AD_CHECK),
	U(AD_FORCE)
};

static struct {
	int cmd;
	int fcn;
} uadmin_opts = {
	A_SHUTDOWN,
	AD_HALT
};

static char uadmin_armed = 0;

#define	NOT_NULL(A) (A == NULL ? "NULL" : A)

static char *
uadmin_cmd2str(int cmd)
{
	int i;
	static char buf[255];

	for (i = 0; i < ARRAY_LEN(uadmin_cmd); i++) {
		if (cmd ==  uadmin_cmd[i].i) {
			return (uadmin_cmd[i].s);
		}
	}
	snprintf(buf, sizeof (buf), "%d", cmd);
	return (buf);
}

static char *
uadmin_fcn2str(int fcn)
{
	int i;
	static char buf[255];

	for (i = 0; i < ARRAY_LEN(uadmin_fcn); i++) {
		if (fcn ==  uadmin_fcn[i].i) {
			return (uadmin_fcn[i].s);
		}
	}
	snprintf(buf, sizeof (buf), "%d", fcn);
	return (buf);
}

static int
init_cmd(void)
{
	int i;
	long l;

	for (i = 0; i < ARRAY_LEN(uadmin_cmd); i++) {
		if (strcasecmp(opts.expert_uadmin_cmd, uadmin_cmd[i].s) == 0) {
			uadmin_opts.cmd = uadmin_cmd[i].i;
			return (1);
		}
	}
	errno = 0;
	l = strtol(opts.expert_uadmin_cmd, NULL, 0);
	if (l != 0 || errno != EINVAL) {
		for (i = 0; i < ARRAY_LEN(uadmin_cmd); i++) {
			if (l == uadmin_cmd[i].i) {
				uadmin_opts.cmd = uadmin_cmd[i].i;
				return (1);
			}
		}
	}

	plog(LOG_ERR,
	    "Unknown uadmin command \"%s\", using default of \"%s\"\n",
	    opts.expert_uadmin_cmd, uadmin_cmd2str(uadmin_opts.cmd));
	return (0);
}

static int
init_fcn(void)
{
	int i;
	long l;

	for (i = 0; i < ARRAY_LEN(uadmin_fcn); i++) {
		if (strcasecmp(opts.expert_uadmin_fcn, uadmin_fcn[i].s) == 0) {
			uadmin_opts.fcn = uadmin_fcn[i].i;
			return (1);
		}
	}
	errno = 0;
	l = strtol(opts.expert_uadmin_fcn, NULL, 0);
	if (l != 0 || errno != EINVAL) {
		for (i = 0; i < ARRAY_LEN(uadmin_fcn); i++) {
			if (l == uadmin_fcn[i].i) {
				uadmin_opts.fcn = uadmin_fcn[i].i;
				return (1);
			}
		}
	}

	plog(LOG_ERR,
	    "Unknown uadmin function \"%s\", using default of \"%s\"\n",
	    opts.expert_uadmin_cmd, uadmin_fcn2str(uadmin_opts.fcn));
	return (0);
}

void
report_uadmin(void)
{
	if (uadmin_armed) {
		plog(LOG_INFO, "uadmin error handler set to: "
		    "uadmin(%s, %s, %s)\n",
		    uadmin_cmd2str(uadmin_opts.cmd),
		    uadmin_fcn2str(uadmin_opts.fcn),
		    NOT_NULL(opts.expert_uadmin_mdep));
	}
}

int
uadmin_init(void)
{
	int c = init_cmd();
	int f = init_fcn();
	uadmin_armed = 1;

	return (f && c);
}

loop_type
on_error_uadmin(ullong_t start, struct aio_str *aiop)
{
	if (uadmin(uadmin_opts.cmd, uadmin_opts.fcn,
	    (intptr_t)opts.expert_uadmin_mdep) < 0) {
		pperror("uadmin(%s, %s, %s)", uadmin_cmd2str(uadmin_opts.cmd),
		    uadmin_fcn2str(uadmin_opts.fcn),
		    NOT_NULL(opts.expert_uadmin_mdep));
	}
	return (CONTINUE);
}
